package de.intevation.gnv.geobackend.base.query;

import java.util.Collection;
import java.util.Iterator;

import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheManager;
import net.sf.ehcache.Element;

import org.apache.log4j.Logger;

import de.intevation.gnv.geobackend.base.Result;
import de.intevation.gnv.geobackend.base.query.cache.CacheCleaner;

/**
 * @author <a href="mailto:sascha.teichmann@intevation.de">Sascha L. Teichmann</a>
 */
public class CachingQueryExecutorFactory
extends      QueryExecutorFactory
{
    public static final String QUERY_EXECUTOR_FACTORY = "caching.query.executor.config";
    public static final String CACHE_NAME             = "sql.cache";

    private static Logger log = Logger.getLogger(CachingQueryExecutorFactory.class);

    protected CacheManager manager;

    public CachingQueryExecutorFactory() {
        log.info("using SQL cache");
        String configFile = System.getProperty(QUERY_EXECUTOR_FACTORY);
        manager = configFile != null
            ? new CacheManager(configFile)
            : new CacheManager();
        manager.addCache(CACHE_NAME);
        CacheCleaner cc = new CacheCleaner();
        cc.start();
    }

    public QueryExecutor getQueryExecutor() {
        return new DefaultQueryExceutor() {

            public Collection<Result> cachedResults(String query) {
                Cache   cache   = manager.getCache(CACHE_NAME);
                Element element = cache.get(query);
                if (log.isDebugEnabled()) {
                    log.debug("found results in SQL cache: " + (element != null));
                }
                return element != null
                    ? (Collection<Result>)element.getObjectValue()
                    : null;
            }

            public void cacheResults(String query, Collection<Result> results) {
                log.debug("store results in SQL cache");
                Cache cache = manager.getCache(CACHE_NAME);
                cache.put(new Element(query, results));
            }
            
            public void clearCache(String[] tableNames) {
                Cache   cache   = manager.getCache(CACHE_NAME);
                Iterator keys = cache.getKeys().iterator();
                while (keys.hasNext()){
                    String origKey = (String)keys.next();
                    String key = origKey.toUpperCase();
                    log.debug(key);
                    for (int i = 0; i < tableNames.length; i++){
                        if (key.contains(tableNames[i])){
                            log.debug(tableNames[i]+ " is contained in "+key);
                            log.debug("Cacheentry will be removed");
                            boolean success = cache.remove(origKey);
                            if (!success){
                                log.warn("Object could not be reoved from Cache.");
                            }
                            break;
                        }
                    }
                }
            }
        };
    }

    public void shutdown() {
        log.info("shutting down SQL cache");
        manager.getCache(CACHE_NAME).flush();
        manager.shutdown();
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
